// Copyright 2023 schukai GmbH
// SPDX-License-Identifier: AGPL-3.0

//go:build !runOnTask

package jobqueue

import (
	"github.com/stretchr/testify/assert"
	"gorm.io/driver/sqlite"
	"gorm.io/gorm"
	"testing"
	"time"
)

func TestWriteToDB1(t *testing.T) {

	gormDB, err := gorm.Open(sqlite.Open("file::memory:?cache=shared"), &gorm.Config{})
	if err != nil {
		t.Fatalf("a error occurred while opening the database: %v", err)
	}

	manager := &Manager{database: gormDB}
	saver := NewDBSaver().SetManager(manager)

	// Starte den DBSaver
	p := StartDBSaver(saver)

	ready := make(chan struct{})

	Then[bool, bool](p, func(value bool) (bool, error) {
		close(ready)
		return value, nil
	}, func(e error) error {
		close(ready)
		Error("Error while starting db saver", "error", err)
		return nil
	})

	<-ready

	runner := &CounterRunnable{}
	job := NewJob[CounterResult]("job1", runner)

	scheduler := &TimeScheduler{
		Time: time.Now().Add(1 * time.Second),
		jobs: nil,
	}

	job.scheduler = scheduler

	err = saver.SaveJob(job)
	assert.Nil(t, err)

	err = saver.SaveJob(job)
	assert.Nil(t, err)

	time.Sleep(1 * time.Second)

	err = saver.Stop()
	assert.Nil(t, err)

}
