//go:build !runOnTask

// Copyright 2023 schukai GmbH
// SPDX-License-Identifier: AGPL-3.0
package jobqueue

import (
	"fmt"
	"github.com/stretchr/testify/assert"
	"gorm.io/driver/sqlite"
	"gorm.io/gorm"
	"testing"
	"time"
)

func TestWriteToDB6(t *testing.T) {

	db, err := gorm.Open(sqlite.Open("file::memory:?cache=shared"), &gorm.Config{})
	if err != nil {
		t.Fatalf("a error occurred while opening the database: %v", err)
	}

	manager := NewManager()
	manager.SetDB(db)
	err = manager.AddWorker(NewLocalWorker(1))
	assert.Nil(t, err)

	err = manager.Start()
	assert.Nil(t, err)

	mgr := manager

	numJobs := 1000
	jobIDs := make([]JobID, numJobs)

	for i := 0; i < numJobs; i++ {
		jobID := JobID(fmt.Sprintf("burstJob%d", i))
		jobIDs[i] = jobID

		runner := &CounterRunnable{}
		job := NewJob[CounterResult](jobID, runner)

		scheduler := &InstantScheduler{}
		err = mgr.ScheduleJob(job, scheduler)
		assert.Nil(t, err)

		err = mgr.dbSaver.SaveJob(job)
		assert.Nil(t, err)
	}

	time.Sleep(10 * time.Second)

	for _, jobID := range jobIDs {
		var tmpJob JobPersistence
		err = db.First(&tmpJob, "id = ?", jobID).Error
		if err != nil {
			t.Errorf("Job id %s not found in database: %v", jobID, err)
		}
	}

	// check if all jobs are in the database
	var jobCount int64
	err = db.Model(&JobPersistence{}).Count(&jobCount).Error
	assert.Nil(t, err)
	assert.Equal(t, int64(numJobs), jobCount)

}
