package jobqueue

import (
	"bytes"
	"log"
	"strings"
	"testing"
)

type MockLogger struct {
	Logs []string
}

func (l *MockLogger) Info(msg string, keysAndValues ...interface{}) {
	l.Logs = append(l.Logs, "INFO: "+msg)
}

func (l *MockLogger) Warn(msg string, keysAndValues ...interface{}) {
	l.Logs = append(l.Logs, "WARN: "+msg)
}

func (l *MockLogger) Error(msg string, keysAndValues ...interface{}) {
	l.Logs = append(l.Logs, "ERROR: "+msg)
}

func (l *MockLogger) Trace(msg string, keysAndValues ...interface{}) {
	l.Logs = append(l.Logs, "DEBUG: "+msg)
}

func TestSetLogger(t *testing.T) {
	mockLogger := &MockLogger{}
	SetLogger(mockLogger)
}

func TestInfo(t *testing.T) {
	mockLogger := &MockLogger{}
	SetLogger(mockLogger)

	Info("test info")

	if len(mockLogger.Logs) != 1 || !strings.HasPrefix(mockLogger.Logs[0], "INFO:") {
		t.Errorf("Info log not as expected")
	}
}

func TestError(t *testing.T) {
	mockLogger := &MockLogger{}
	SetLogger(mockLogger)

	Error("test error")

	if len(mockLogger.Logs) != 1 || !strings.HasPrefix(mockLogger.Logs[0], "ERROR:") {
		t.Errorf("Error log not as expected")
	}
}

func TestWarn(t *testing.T) {
	mockLogger := &MockLogger{}
	SetLogger(mockLogger)

	Warn("test warning")

	if len(mockLogger.Logs) != 1 || !strings.HasPrefix(mockLogger.Logs[0], "WARN:") {
		t.Errorf("Warn log not as expected")
	}
}

func TestDebug(t *testing.T) {
	mockLogger := &MockLogger{}
	SetLogger(mockLogger)

	Trace("test debug")

	if len(mockLogger.Logs) != 1 || !strings.HasPrefix(mockLogger.Logs[0], "DEBUG:") {
		t.Errorf("Debug log not as expected")
	}
}

func TestDefaultLogger(t *testing.T) {
	buffer := bytes.NewBufferString("")
	SetLogger(&DefaultLogger{Logger: log.New(buffer, "", log.LstdFlags)})

	message := "Hello World"
	Info(message)

	if !strings.Contains(buffer.String(), message) {
		t.Errorf("Expected %v in the output logs", message)
	}
}
